/*  Copyright (C) 2000, 2006  Marc Toussaint (mtoussai@inf.ed.ac.uk)
    under the terms of the GNU LGPL (http://www.gnu.org/copyleft/lesser.html)
    see the `std.h' file for a full copyright statement  */

/*! \file array.h
    \brief core file: defines the MT::Array class */
#ifndef MT_array_h
#define MT_array_h

//#define MT_EXPRESSIONS

#include"std.h"

#define FOR1D(x,i) for(i=0;i<x.d0;i++)
#define FOR2D(x,i,j) for(i=0;i<x.d0;i++) for(j=0;j<x.d1;j++)

#define forAll(i,A) for(i=A.p;i!=A.pstop;i++)

//===========================================================================
//
// handle expressions of arrays more efficiently -- PRELIMINARY
//

#ifdef MT_EXPRESSIONS
namespace MT{
enum ExOp { UNI, PLUS, MINUS, MUL, Div, PROD, OUT };
struct Ex{
  ExOp op;
  double add,mul;
  bool trans;
  void *A,*B;
  Ex(MT::ExOp OP,void *AA,void *BB){ op=OP; A=AA; B=BB; }
  Ex(void *_A,double _add=0,double _mul=1,bool _trans=false){ op=UNI; A=_A; add=_add; mul=_mul; trans=_trans; }
};
}
#  define IFEX(x) x
#else
#  define IFEX(x)
#endif


//===========================================================================
//
// Array class
//

namespace MT{
/*! Simple array container to store 1-, 2-, 3-dimensional arrays; can
    buffer more memory than necessary for faster resize; enables
    non-const reference of subarrays; enables fast memove for
    elementary types. Please see also the reference for the \ref
    array.h header, which contains lots of functions that can be
    applied on Arrays. */
template<class T>
class Array{
public:
  T *p;	    //!< the pointer on the linear memory allocated
  T *pstop; //!< end of memory (pstop is already out of the bound)
  uint N;   //!< number of elements
  uint nd;  //!< number of dimensions
  uint d0;  //!< 0th dim
  uint d1;  //!< 1st dim
  uint d2;  //!< 2nd dim
  uint memSize;//!< size of allocated memory (may be greater than N)
  bool reference;//!< true if this refers to some external memory
  T **pp;   //!< C-style 2D pointer (only valid if created with \c getCarray!)
  IFEX( Ex *ex; )
  bool temp;

public://!@name Options
  /*!\brief be careful!!!; this option will modify the resizeCopy(...):
      instead of calling the copy operator= for each element, the
      memory is moved with the \c memmove command - works only if you
      know what you do ... [default is true for simple types and false
      for unknown ones] */
  bool memMove;

  /*!\brief if flexiMem is true (which is default!) the resize method will
      (1) at the first call allocate the exact amount of memory, (2)
      at further calls of increasing memory allocate twice the memory
      needed or (3) at further calls of decreasing memory only free
      the memory if the new size is smaller than a fourth */
  bool flexiMem;
  //@}
private:
  static char memMoveInit;
  static int sizeT;
  void init(){
    reference=false;
    memMove=false;
    if(sizeT==-1) sizeT=sizeof(T);
    if(memMoveInit==-1){
      memMoveInit=0;
      if(typeid(T)==typeid(bool) ||
	typeid(T)==typeid(char) ||
	typeid(T)==typeid(unsigned char) ||
	typeid(T)==typeid(int) ||
	typeid(T)==typeid(unsigned int) ||
	typeid(T)==typeid(short) ||
	typeid(T)==typeid(unsigned short) ||
	typeid(T)==typeid(long) ||
	typeid(T)==typeid(unsigned long) ||
	typeid(T)==typeid(float) ||
	typeid(T)==typeid(double)) memMoveInit=1;
    }
    memMove=(memMoveInit==1);
    flexiMem=true;
    p=pstop=0;
    memSize=N=nd=d0=d1=d2=0;
    pp=0;
    IFEX( ex=0; );
    temp=false;
  }

public://!@name Constructors

  //! standard constructor -- this becomes an empty array
  Array(){ init(); }

  //! copy constructor
  Array(const Array<T>& a){ init(); operator=(a); }

  //! constructor with resize
  Array(uint i){ init(); resize(i); }

  //! constructor with resize
  Array(uint i,uint j){ init(); resize(i,j); }

  //! constructor with resize
  Array(uint i,uint j,uint k){ init(); resize(i,j,k); }

  //! this becomes a reference on a subdimension of \c a
  Array(const Array<T>& a,uint dim){ init(); referToSubDim(a,dim); }

  //! this becomes a reference on a subdimension of \c a
  Array(const Array<T>& a,uint i,uint j){ init(); referToSubDim(a,i,j); }

  //! this becomes a reference on the C-array \c p
  Array(T* p,uint size){ init(); referTo(p,size); }

  //! initialize the array from a string (with operator<<)
  Array(const char* str){ init(); set(str); }

  IFEX( Array(Ex *_ex){ ex=_ex; } )

  ~Array(){
    IFEX( if(ex){ delete ex; return; } )
    freeMEM();
  }
  

public://!@name Resize

  //! same as resize(0);
  void clear(){ resize(0); }

  //! 1D resize
  void resize(uint D0){ nd=1; d0=D0; resizeMEM(d0,false); }
  void resizeCopy(uint D0){ nd=1; d0=D0; resizeMEM(d0,true); }
  void resizeZero(uint D0){ nd=1; d0=D0; resizeMEM(d0,false); setZero(); }
  void reshape(uint D0){ CHECK(N==D0,"reshape must preserve total memory size"); nd=1; d0=D0; d1=d2=0; }

  //! 2D resize
  void resize(uint D0,uint D1){ nd=2; d0=D0; d1=D1; resizeMEM(d0*d1,false); }
  void resizeCopy(uint D0,uint D1){ nd=2; d0=D0; d1=D1; resizeMEM(d0*d1,true); }
  void reshape(uint D0,uint D1){ CHECK(N==D0*D1,"reshape must preserve total memory size"); nd=2; d0=D0; d1=D1; d2=0; }

  //! 3D resize
  void resize(uint D0,uint D1,uint D2){ nd=3; d0=D0; d1=D1; d2=D2; resizeMEM(d0*d1*d2,false); }
  void resizeCopy(uint D0,uint D1,uint D2){ nd=3; d0=D0; d1=D1; d2=D2; resizeMEM(d0*d1*d2,true); }
  void reshape(uint D0,uint D1,uint D2){ CHECK(N==D0*D1*D2,"reshape must preserve total memory size"); nd=3; d0=D0; d1=D1; d2=D2; }

#ifndef MT_MSVC
  //! make it the same size as \c a
  void resize(const Array<T>& a){ nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2; resizeMEM(a.N,false); }
  void resizeCopy(const Array<T>& a){ nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2; resizeMEM(a.N,true); }
#endif

  //! resize to the same size as \c a
  template<class S>
  void resize(const Array<S>& a){ nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2; resizeMEM(a.N,false); }
  template<class S>
  void resizeCopy(const Array<S>& a){ nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2; resizeMEM(a.N,true); }
  //@}

private: //core rountine to allocate memory
  void resizeMEM(uint size,bool copy){
    if(reference){
      CHECK(size==N,"real resize of subarray is not allowed! (only a resize without changing memory size)");
      return;
    }
    if(N==size) return;
    uint i;
    T *pold=p;
    if(!flexiMem){
      if(memSize!=size){
	if(size) p=new T [size];
	if(!p) HALT("MT::Array failed memory allocation of "<<size*sizeT<<"bytes");
	if(copy && !memMove) for(i=N<size?N:size;i--;) p[i]=pold[i];
	if(copy && memMove) memmove(p,pold,sizeT*(N<size?N:size));
	if(memSize) delete[] pold;
	N=memSize=size;
	pstop=p+N;
      }
    }else{
      if(size>0 && memSize==0){ //first time
	p=new T [size];
	if(!p) HALT("MT::Array failed memory allocation of "<<size*sizeT<<"bytes");
	memSize=size;
      }else if(size>memSize || 10+2*size<memSize/2){
	uint oversize=10+2*size;
	p=new T [oversize];
	if(!p) HALT("MT::Array failed memory allocation of "<<size*sizeT<<"bytes");
	if(copy && !memMove) for(i=N<size?N:size;i--;) p[i]=pold[i];
	if(copy && memMove) memmove(p,pold,sizeT*(N<size?N:size));
	if(memSize) delete[] pold;
	memSize=oversize;
      }
      N=size;
      pstop=p+N;
    }
    if(pp){ delete[] pp; pp=0; }
  }
  void freeMEM(){
    if(memSize) delete[] p;
    if(pp) delete[] pp;
    p=pstop=0;
    memSize=N=nd=d0=d1=d2=0;
  }


public://!@name append, insert & remove

  //! append an element to the array -- the array becomes 1D!
  void append(const T& x){
    if(nd==2 && d1==1)
      resizeCopy(d0+1,d1);
    else
      resizeCopy(N+1);
    p[N-1]=x;
  }
  //! append another array to the array (by copying it) -- the array might become 1D!
  void append(const Array<T>& x){
    uint oldN=N,i;
    if(nd==2 && d1==x.N) 
      resizeCopy(d0+1,d1);
    else
      resizeCopy(N+x.N);
    if(!memMove){
      for(i=0;i<x.N;i++) p[oldN+i]=x.p[i];
    }else memmove(p+oldN,x.p,sizeT*x.N);
  }
  //! append a C array to the array (by copying it) -- the array might become 1D!
  void append(const T *p,uint n){
    uint oldN=N,i;
    if(nd==2 && d1==n) 
      resizeCopy(d0+1,d1);
    else
      resizeCopy(N+n);
    if(!memMove){
      for(i=0;i<n;i++) p[n+i]=p[i];
    }else memmove(p+oldN,p,sizeT*n);
  }

  /* inserts x at the position i -- the array becomes 1D! [only with memMove!] */
  void insert(uint i,const T& x){
    CHECK(memMove,"only with memMove");
    uint Nold=N;
    resizeCopy(Nold+1);
    if(i<Nold) memmove(p+i+1,p+i,sizeT*(Nold-i));
    p[i]=x;
  }

  //! remove (delete) a subsequence of the array -- the array becomes 1D!  [only with memMove!]
  void remove(uint i,uint n=1){
    CHECK(memMove,"only with memMove");
    if(N>i+n) memmove(p+i,p+i+n,sizeT*(N-i-n));
    resizeCopy(N-n);
  }

  //! replace n elements at pos i by the sequence x -- the array becomes 1D!  [only with memMove!]
  void replace(uint i,uint n,const Array<T>& x){
    CHECK(memMove,"only with memMove");
    uint Nold=N;
    if(n==x.N){
      memmove(p+i,x.p,sizeT*(x.N));
    }else if(n>x.N){
      memmove(p+i+x.N,p+i+n,sizeT*(Nold-i-n));
      if(i+n<Nold) memmove(p+i,x.p,sizeT*(x.N));
      resizeCopy(Nold-n+x.N);
    }else{
      resizeCopy(Nold+x.N-n);
      if(i+n<Nold) memmove(p+i+x.N,p+i+n,sizeT*(Nold-i-n));
      memmove(p+i,x.p,sizeT*(x.N));
    }
  }

  //! deletes the i-th row
  void delRow(uint i){
    CHECK(memMove,"only with memMove");
    CHECK(i<d0,"range check error");
    uint n=d1;
    if(i+1<d0) memmove(p+i*n,p+(i+1)*n,sizeT*(d0-i-1)*n);
    resizeCopy(d0-1,n);
  }

  //! deletes the i-th column
  void delColumn(uint i){
    CHECK(memMove,"only with memMove");
    CHECK(i<d1,"range check error");
    uint n=d1;
    for(uint j=0;j<d0;j++){
      memmove(p+j*(n-1)  ,p+j*n  ,sizeT*i);
      memmove(p+j*(n-1)+i,p+j*n+(i+1),sizeT*(n-i-1));
    }
    resizeCopy(d0,n-1);
  }
  


public://!@name Access operations
  //! the \c ith element
  T& elem(uint i) const{
    CHECK(i<N,"range error ("<<i<<">="<<N<<")");
    return p[i];
  }

  /* scalar reference (legal iff N==1) */
  /*operator T&() const{ 
    CHECK(N==1,"scalar reference ("<<N<<"!=1)");
    return *e; }*/

  //! 1D reference
  T& operator()(uint i) const{ 
    CHECK(nd==1 && i<d0,"1D range error ("<<nd<<"=1,"<<i<<"<"<<d0<<")");
    return p[i]; }

  //! 2D reference
  T& operator()(uint i,uint j) const{
    CHECK(nd==2 && i<d0 && j<d1,
	  "2D range error ("<<nd<<"=2,"<<i<<"<"<<d0<<","<<j<<"<"<<d1<<")");
    return p[i*d1+j];
  }

  //! 3D reference
  T& operator()(uint i,uint j,uint k) const{
    CHECK(nd==3 && i<d0 && j<d1 && k<d2,
	  "3D range error ("<<
	  nd<<"=3,"<<i<<"<"<<d0<<","<<j<<"<"<<d1<<","<<k<<"<"<<d2<<")");
    return p[(i*d1+j)*d2+k];
  }

  //! get a subarray; use in conjuction with operator()() to get a reference
  Array<T> operator[](uint i) const{ return Array(*this,i); }

  //! convert a subarray into a reference (e.g. a[3]()+=.123)
  Array<T>& operator()(){ return (*this); }

  /*!\brief the index of the maxium; precondition: the comparision operator
      > exists for type T */ 
  uint maxIndex(){ uint i,m=0; for(i=0;i<N;i++) if(p[i]>p[m]) m=i; return m; }

  /*!\brief the index of the maxium; precondition: the comparision operator
      > exists for type T */ 
  void maxIndex(uint& i,uint& j){ CHECK(nd==2,"needs 2D array"); j=maxIndex(); i=j/d1; j=j%d1; }

  /*!\brief the index of the maxium; precondition: the comparision operator
      > exists for type T */ 
  void maxIndex(uint& i,uint& j,uint& k){ CHECK(nd==3,"needs 3D array"); k=maxIndex(); i=k/(d1*d2); k=k%(d1*d2); j=k/d2; k=k%d2; }

  //! ...
  uint minIndex(){ uint i,m=0; for(i=0;i<N;i++) if(p[i]<p[m]) m=i; return m; }

  bool contains(T& x){ uint i; for(i=0;i<N;i++) if(p[i]==x) return true; return false; }

  /*!\brief a sub array of a 1D Array (corresponds to matlab [i:I]); when
      the upper limit I is -1, it is replaced by the max limit (like
      [i:]) */
  Array<T> sub(uint i,int I) const{
    CHECK(nd==1,"1D range error ");
    Array<T> x;
    if(I==-1) I=d0-1;
    CHECK(i<=I,"lower limit higher than upper!");
    x.resize(I-i+1);
    uint k;
    for(k=i;k<=I;k++) x(k-i)=operator()(k);
    return x;
  }

  /*!\brief copies a sub array of a 2D Array (corresponds to matlab [i:I,j:J]);
      when the upper limits I or J are -1, they are replaced by the
      max limit (like [i:,j:]) */
  Array<T> sub(uint i,int I,uint j,int J) const{
    CHECK(nd==2,"2D range error ");
    Array<T> x;
    if(I==-1) I=d0-1;
    if(J==-1) J=d1-1;
    CHECK(i<(uint)I && j<(uint)J,"lower limit must be higher than upper!");
    x.resize(I-i+1,J-j+1);
    uint k,l;
    for(k=i;k<=(uint)I;k++) for(l=j;l<=(uint)J;l++) x(k-i,l-j)=operator()(k,l);
    return x;
  }

  /*!\brief copies a selection of columns from a 2D array, the first index (rows)
    runs from i to I (as explained above) while the second index runs
    over the columns explicitly referred to by cols. (col doesn't have
    to be ordered or could also contain some columns multiply) */
  Array<T> sub(uint i,int I,Array<uint> cols) const{
    CHECK(nd==2,"2D range error ");
    Array<T> x;
    if(I==-1) I=d0-1;
    CHECK(i<=(uint)I,"lower limit higher than upper!");
    x.resize(I-i+1,cols.N);
    uint k,l;
    for(k=i;k<=(uint)I;k++) for(l=0;l<cols.N;l++) x(k-i,l)=operator()(k,cols(l));
    return x;
  }
  
public://!@name C-array Access

  // allocates, sets and returns the pp-pointer (of type T**)
  T** getCarray(){
    CHECK(nd==2,"only 2D array gives C-array of type T**");
    if(pp) return pp;
    pp=new T* [d0];
    for(uint i=0;i<d0;i++) pp[i]=p+i*d1;
    return pp;
  }

  /*!\brief returns an ordinary 2-dimensional C-pointer to the Array content.
      Requires the Array<T*> as buffer. */
  T** getPointers(Array<T*>& array2d) const{
    CHECK(nd==2,"only 2D array gives C-array of type T**");
    array2d.resize(d0);
    for(uint i=0;i<d0;i++) array2d(i)=p+i*d1;
    return array2d.p;
  }

  //! returns an ordinary 3-dimensional C-pointer-array
  T*** getPointers(Array<T**>& array3d,Array<T*>& array2d) const{
    CHECK(nd==3,"only 3D array gives C-array of type T*** ");
    array2d.resize(d0,d1);
    for(uint i=0;i<d0;i++){
      for(uint j=0;j<d1;j++) array2d(i,j)=&operator()(i,j,0);
      array3d(i)=&array2d(i,0);
    }
    return array3d.p;
  }



public://!@name Assignments

  //! set all elements to value \c v 
  Array<T>& operator=(const T v){
    uint i;
    //if(memMove && typeid(T)==typeid(T)) memset(p,*((int*)&v),N); else
    for(i=0;i<N;i++) p[i]=v;
    return *this;
  }

  //! copy operator
  Array<T>& operator=(const Array<T>& a){
    IFEX( if(a.ex){ assignEx(a); return *this; } );
    //if(a.temp){ takeOver(*((Array<T>*)&a)); return *this; }
    resize(a);
    uint i;
    if(memMove) memmove(p,a.p,sizeT*N);
    else for(i=0;i<N;i++) p[i]=a.p[i];
    return *this;
  }

  //! reads it from the C string (by using std::istringstream)
  void set(const char* str){
    std::istringstream s(str);
    readRaw(s);
  }

#ifdef MT_EXPRESSIONS
  void checkEx() const{ if(ex) assignEx(); }
  void assignEx(const Array<T>& a) const;
  void assignEx() const;
  //Array<T>& operator=(const Ex& ex);
#else
  void checkEx() const{}
#endif

  //! generic copy; precondition: operator (T)=(S) exists
  template<class S>
  Array<T>& copy(const Array<S>& a){
    resize(a);
    uint i;
    if(memMove && typeid(T)==typeid(S)) memmove(p,a.p,sizeT*N);
    else for(i=0;i<N;i++) p[i]=(T)a.p[i];
    return *this;
  }

  /*!\brief same as memset(p,0,sizeT*N); precondition: memMove is
      true! */
  void setZero(byte zero=0){
    CHECK(memMove,"can set array's memory to zero only if memMove option is true");
    memset(p,zero,sizeT*N);
  }

  //! copies the ordinary C buffer into this array
  template<class S>
  void copy(S *buffer,uint D0){
    resize(D0);
    uint i;
    if(memMove && typeid(T)==typeid(S))
      memmove(p,buffer,sizeT*d0);
    else for(i=0;i<d0;i++) operator()(i)=(T)buffer[i];
  }

  //! copies the ordinary C buffer into this array
  template<class S>
  void copy(S **buffer,uint D0,uint D1){
    resize(D0,D1);
    uint i,j;
    for(i=0;i<d0;i++){
      if(memMove && typeid(T)==typeid(S))
	memmove(p+i*d1,buffer[i],sizeT*d1);
      else for(j=0;j<d1;j++) operator()(i,j)=(T)buffer[i][j];
    }
  }

  /*!\brief copies this array into an ordinary C buffer !!be sure that
      buffer has the right size!! */
  template<class S>
  void copyInto(S *buffer){
    CHECK(nd==1,"can only copy 1D Array into 1D C-array");
    uint i;
    if(memMove && typeid(T)==typeid(S)) memmove(buffer,p,sizeT*d0);
    else for(i=0;i<d0;i++) buffer[i]=(S)operator()(i);
  }

  /*!\brief copies this array into an ordinary C buffer !!be sure that
      buffer has the right size!! */
  template<class S>
  void copyInto2D(S **buffer){
    CHECK(nd==2,"can only copy 2D Array into 2D C-array");
    uint i,j;
    for(i=0;i<d0;i++){
      if(memMove && typeid(T)==typeid(S)) memmove(buffer[i],p+i*d1,sizeT*d1);
      else for(j=0;j<d1;j++) buffer[i][j]=(S)operator()(i,j);
    }
  }

  //! makes this array a reference to the C buffer
  void referTo(const T *buffer,uint n){
    freeMEM();
    reference=true;
    nd=1; d0=n; d1=d2=0; N=n;
    p=(T*)buffer;
    pstop=p+N;
  }

  //! make this array a reference to the array \c a
  void referTo(const Array<T>& a){
    freeMEM();
    reference=true; memMove=a.memMove; flexiMem=a.flexiMem;
    N=a.N; nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2;
    p=a.p;
    pstop=a.pstop;
  }

  //! make this array a subarray reference to \c a
  void referToSubRange(const Array<T>& a,uint i,int I){
    CHECK(a.nd==1,"not implemented yet");
    freeMEM();
    reference=true; memMove=a.memMove; flexiMem=a.flexiMem;
    if(I==-1) I=a.N-1;
    N=I+1-i; nd=1; d0=N; d1=0; d2=0;
    p=a.p+i;
    pstop=p+N;
  }

  //! make this array a subarray reference to \c a
  void referToSubDim(const Array<T>& a,uint dim){
    CHECK(a.nd>1,"can't create subarray of array less than 2 dimensions");
    freeMEM();
    reference=true; memMove=a.memMove; flexiMem=a.flexiMem;
    if(a.nd==2){
      nd=1; d0=a.d1; d1=d2=0; N=d0;
      p=&a(dim,0);
      pstop=p+N;
    }
    if(a.nd==3){
      nd=2; d0=a.d1; d1=a.d2; d2=0; N=d0*d1;
      p=&a(dim,0,0);
      pstop=p+N;
    }
  }

  //! make this array a subarray reference to \c a
  void referToSubDim(const Array<T>& a,uint i,uint j){
    CHECK(a.nd>2,"can't create subsubarray of array less than 3 dimensions");
    freeMEM();
    reference=true; memMove=a.memMove; flexiMem=a.flexiMem;
    if(a.nd==3){
      nd=1; d0=a.d2; d1=0; d2=0; N=d0;
      p=&a(i,j,0);
      pstop=p+N;
    }
  }

  //! takes over the memory buffer from a; afterwards, a is only a reference on the memory
  void takeOver(Array<T>& a){
    freeMEM();
    memMove=a.memMove; flexiMem=a.flexiMem;
    N=a.N; nd=a.nd; d0=a.d0; d1=a.d1; d2=a.d2;
    p=a.p;
    pstop=a.pstop;
    memSize=a.memSize;
    a.reference=true;
    a.memSize=0;
  }


public://!@name stuff for generic looping
  void First(T*& i) const{ i=p; }
  void Last(T*& i) const{ i=(p+N)-1; }
  void Next(T*& i) const{ i++; }
  bool CheckNext(T*& i) const{ return i!=pstop; }
  void Prev(T*& i) const{ i--; }
  bool CheckPrev(T*& i) const{ return i!=p-1; }


public://!@name permutations
  //! permute the elements \c i and \c j
  void permute(uint i,uint j){ T x=p[i]; p[i]=p[j]; p[j]=x; }

  //! permute all elements randomly
  void randomPermute(){
    int j,r;
    for(j=N-1;j>=1;j--){
      r=rnd(j+1);
      permute(r,j);
    }
  }


public://!@name I/O

  //! prototype for operator<<
  void write(std::ostream& os,char *ARRAYSEP=" ",bool binary=false) const{
    CHECK(!binary || memMove,"binary write works only for memMoveable data");
    uint i,j,k;
    if(!IOraw){//write tag
      if(nd==0){ os <<"<0>"; return; }
      if(nd==1)  os <<"<1:" <<d0;
      if(nd==2)  os <<"<2:" <<d0 <<',' <<d1;
      if(nd==3)  os <<"<3:" <<d0 <<',' <<d1 <<',' <<d2;
    }
    //write data
    if(binary){
      os.write((char*)p,sizeT*N);
    }else{
      if(nd==1){
	//os <<' ';
	for(i=0;i<d0;i++) os <<ARRAYSEP <<operator()(i);
      }
      if(nd==2) for(j=0;j<d0;j++){
	if(j || !IOraw) os <<"\n";
	for(i=0;i<d1;i++) os <<ARRAYSEP <<operator()(j,i);
      }
      if(nd==3) for(k=0;k<d0;k++){
        if(k || !IOraw) os <<"\n";
        for(j=0;j<d1;j++){
          for(i=0;i<d2;i++) os <<ARRAYSEP <<operator()(k,j,i);
          os <<"\n";
        }
      }
    }
    if(!IOraw){//write end tag
      os <<">";
      if(nd>1) os <<std::endl;
    }
  }

  //! prototype for operator>>
  void read(std::istream& is,bool binary=false){
    CHECK(!binary || memMove,"binary write works only for memMoveable data");
    uint d,i,j,k;
    if(!IOraw){//read tag
      is >>"<" >>d;
      if(is.fail()) HALT ("could not read array tag");
      if(d==0){ is >>">"; return; }
      if(d==1){
        is >>":" >>i;
        if(is.fail()) HALT ("could not read array's dimensions");
	resize(i);
      }
      if(d==2){
        is >>":" >>i >>"," >>j;
	if(is.fail()) HALT ("could not read array's dimensions");
        resize(i,j);
      }
      if(d==3){
	is >>":" >>i >>"," >>j >>"," >>k;
	if(is.fail()) HALT ("could not read array's dimensions");
        resize(i,j,k);
      }
    }
    //read data
    if(binary){
      is.read((char*)p,sizeT*N);
    }else{
      for(i=0;i<N;i++){
	if(is.fail()) HALT("could not read "<<i<<"-th element of an array");
	is >>p[i];
      }
    }
    //read end tag
    if(!IOraw){
      is >>">";
      if(is.fail()) HALT ("could not read array end tag");
    }
  }

  //! IO-manip: arrays will be streamed as raw (without tags)
  const Array<T>& ioraw() const{ IOraw=true; return *this; }

  //! IO-manip: arrays will be streamed non-raw (with tags)
  const Array<T>& ionoraw() const{ IOraw=false; return *this; }

  //! read array from stream without knowing yet the dimensionalities
  void readRaw(std::istream& is,uint d1=0,uint d2=0){
    resize(0);
    T x;
    for(;;){
      MT::skip(is);
      if(!is.good()) break;
      is >>x;
      if(is.fail()) HALT("error when reading "<<N<<"th element in readRaw");
      if(d1 && d2 && N>=d1*d2) break;
      append(x);
    }
    if(d1 && N>d1){
      CHECK(!(N%d1),"read "<<N<<"elements - not dividable by "<<d1);
      resizeCopy(N/d1,d1);
    }
  }
  void readRaw(const char* filename,uint d1=0,uint d2=0){
    std::ifstream is; MT::open(is,filename); readRaw(is,d1,d2);
  }
  void writeRaw(const char* filename){
    std::ofstream os; MT::open(os,filename); os <<(*this).ioraw();
  }
  //@}

#ifndef MT_EXPRESSIONS
  //! transpose
  friend inline Array<T> operator~(const Array<T>& y){ Array<T> x; transpose(x,y); x.temp=true; return x; }
  //! inner product
  friend inline Array<T> operator*(const Array<T>& y,const Array<T>& z){ Array<T> x; innerProduct(x,y,z); x.temp=true; return x; }
  //! element-wise multiplication (sorry for the awkward notation!!)
  friend inline Array<T> operator%(const Array<T>& y,const Array<T>& z){ Array<T> x; mult(x,y,z); x.temp=true; return x; }
  //! outer product
  friend inline Array<T> operator^(const Array<T>& y,const Array<T>& z){ Array<T> x; outerProduct(x,y,z); x.temp=true; return x; }
  //! scalar multiplication
  friend inline Array<T> operator*(const Array<T>& y,T a){ Array<T> x; scalarMultiplication(x,y,a); x.temp=true; return x; }
  //! scalar multiplication
  friend inline Array<T> operator*(T a,const Array<T>& y){ Array<T> x; scalarMultiplication(x,y,a); x.temp=true; return x; }
#endif
};
}


//===========================================================================
//
//!@name basic operators
//

//! calls MT::Array<T>::read
template<class T> inline std::istream& operator>>(std::istream& is,MT::Array<T>& x){ x.read(is);return is; }

//! calls MT::Array<T>::write
template<class T> inline std::ostream& operator<<(std::ostream& os,const MT::Array<T>& x){
  IFEX( if(x.ex) x.assignEx(); )
  x.write(os); return os;
}

//! check whether this and \c a have same dimensions
template<class T,class S>
inline bool samedim(const MT::Array<T>& a,const MT::Array<S>& b){
  return (b.nd==a.nd && b.d0==a.d0 && b.d1==a.d1 && b.d2==a.d2);
}

//! equal in size and all elements
template<class T>
inline bool operator==(const MT::Array<T>& v,const MT::Array<T>& w){
  if(!samedim(v,w)) return false;
  const T *iv,*iw;
  for(iv=v.p,iw=w.p; iv!=v.pstop; iv++,iw++)
    if (*iv != *iw) return false;
  return true;
}

//! not equal
template<class T>
inline bool operator!=(const MT::Array<T>& v,const MT::Array<T>& w){
  return !(v==w);
}

//! lexical comparison
template<class T>
inline bool operator<(const MT::Array<T>& v,const MT::Array<T>& w){
  if(v.N==w.N){
    for(uint i=0;i<v.N;i++){
      if(v.p[i]>w.p[i]) return false;
      if(v.p[i]<w.p[i]) return true;
    }
    return false; //they are equal
  }
  return v.N<w.N;
}


//===========================================================================
//
//!@name standard arithmetic operators
//

#ifndef MT_doxy // exclude these macros when generating the documentation
                // (doxygen can't handle them...)

//---------- unary operators

#define UnaryOperator( op )                                               \
template<class T>                                                         \
inline MT::Array<T> operator op (const MT::Array<T>& y){                  \
  MT::Array<T> x;                                                         \
  x.resize(y);								  \
  T *ix=x.p;                                                              \
  const T *iy=y.p;                                                        \
  for(; iy!=y.pstop; iy++,ix++) *ix= op *iy;			  	  \
  x.temp=true; return x;                                                  \
}

//UnaryOperator( ! )
//UnaryOperator( ~ )
//UnaryOperator( + )
UnaryOperator( - )
#undef UnaryOperator


//---------- binary operators

#ifndef MT_EXPRESSIONS
#define BinaryOperator( op )                                              \
template<class T>                                                         \
inline MT::Array<T> operator op(const MT::Array<T>& v,const MT::Array<T>& w){ \
  v.checkEx(); w.checkEx(); \
  CHECK(v.N==w.N,							  \
  "binary operator on different array dimensions ("<<v.N<<", "<<w.N<<")");\
  MT::Array<T> z;                                                         \
  z.resize(v);								  \
  T *iz=z.p;								  \
  const T *iw=w.p,*iv=v.p;						  \
  for(; iv!=v.pstop; iv++,iw++,iz++) *iz = *iv op *iw;                    \
  z.temp=true; return z;                                                               \
}                                                                         \
                                                                          \
template<class T>                                                         \
inline MT::Array<T> operator op (const MT::Array<T>& v,T w){              \
  v.checkEx(); \
  MT::Array<T> z;                                                         \
  z.resize(v);								  \
  T *iz=z.p;								  \
  const T *iv=v.p;							  \
  for(; iv!=v.pstop; iv++,iz++) *iz = *iv op w;                           \
  z.temp=true; return z;                                                               \
}                                                                         \
                                                                          \
template<class T>                                                         \
inline MT::Array<T> operator op ( T v, const MT::Array<T>& w ){           \
  w.checkEx(); \
  MT::Array<T> z;                                                         \
  z.resize(w);								  \
  T *iz=z.p;								  \
  const T *iw=w.p;							  \
  for(; iw!=w.pstop; iw++,iz++) *iz = v op *iw;                           \
  z.temp=true; return z;                                                               \
}

BinaryOperator( || )
BinaryOperator( && )
BinaryOperator( | )
  //BinaryOperator( ^ ) I reserve this for the outer product!
BinaryOperator( & )
#ifndef MT_EXPRESSIONS
BinaryOperator( + )
BinaryOperator( - )
#endif
  //BinaryOperator( * ) I reserve this operator for the inner product!
BinaryOperator( / )
BinaryOperator( % )
#undef BinaryOperator
#endif


//---------- compound assignment operators

#define CompoundAssignmentOperator( op )                                  \
template<class T,class S>                                                 \
inline MT::Array<T>& operator op (MT::Array<T>& x,const MT::Array<S>& y){ \
  y.checkEx(); \
  CHECK(x.N==y.N,							  \
  "binary operator on different array dimensions ("<<x.N<<", "<<y.N<<")");\
  T *ix=x.p;								  \
  const S *iy=y.p;							  \
  for(; ix!=x.pstop; ix++,iy++) *ix op *iy;                               \
  x.temp=true; return x;                                                               \
}                                                                         \
                                                                          \
template<class T>                                                         \
inline MT::Array<T>& operator op ( MT::Array<T>& x, T y ){                \
  T *ix=x.p;								  \
  for(; ix!=x.pstop; ix++) *ix op y;                                      \
  x.temp=true; return x;                                                               \
}                                                                         \

CompoundAssignmentOperator( |= )
CompoundAssignmentOperator( ^= )
CompoundAssignmentOperator( &= )
CompoundAssignmentOperator( += )
CompoundAssignmentOperator( -= )
CompoundAssignmentOperator( *= )
CompoundAssignmentOperator( /= )
CompoundAssignmentOperator( %= )
#undef CompoundAssignmentOperator


//---------- unary functions

#define UnaryFunction( func )                                             \
template<class T>                                                         \
inline MT::Array<T> func (const MT::Array<T>& v){                         \
  v.checkEx(); \
  MT::Array<T> z;                                                         \
  z.resize(v);                                                            \
  T *iz=z.p;								  \
  const T *iv=v.p;							  \
  for(; iv!=v.pstop; iv++, iz++) *iz = ::func( *iv );                     \
  z.temp=true; return z;                                                               \
}									  \

// trigonometric functions
UnaryFunction( acos )
UnaryFunction( asin )
UnaryFunction( atan )
UnaryFunction( cos )
UnaryFunction( sin )
UnaryFunction( tan )

// hyperbolic functions
UnaryFunction( cosh )
UnaryFunction( sinh )
UnaryFunction( tanh )
UnaryFunction( acosh )
UnaryFunction( asinh )
UnaryFunction( atanh )

// exponential and logarithmic functions
UnaryFunction( exp )
UnaryFunction( log )
UnaryFunction( log10 )

// power functions
UnaryFunction( sqrt )
UnaryFunction( cbrt )

// nearest integer and absolute value
UnaryFunction( ceil )
UnaryFunction( fabs )
UnaryFunction( floor )
#undef UnaryFunction


//---------- binary functions

#define BinaryFunction( func )                                            \
template<class T>                                                         \
inline MT::Array<T> func(const MT::Array<T>& v,const MT::Array<T>& w){    \
  v.checkEx(); w.checkEx(); \
  CHECK(v.N==w.N,							  \
  "binary operator on different array dimensions ("<<v.N<<", "<<w.N<<")");\
  MT::Array<T> z;                                                         \
  z.resize(v);                                                            \
  for(uint i=z.N;i--; ) z.p[i]= func(v.p[i],w.p[i]);			  \
  z.temp=true; return z;                                                               \
}                                                                         \
                                                                          \
template<class T>                                                         \
inline MT::Array<T> func(const MT::Array<T>& v,T w){                      \
  v.checkEx(); \
  MT::Array<T> z;                                                         \
  z.resize(v);                                                            \
  for(uint i=z.N;i--; ) z.p[i]= func(v.p[i],w);			  \
  z.temp=true; return z;                                                               \
}                                                                         \
                                                                          \
template<class T>                                                         \
inline MT::Array<T> func(T v,const MT::Array<T>& w){                      \
  w.checkEx(); \
  MT::Array<T> z;                                                         \
  z.resize(w);                                                            \
  for(uint i=z.N;i--; ) z.p[i]= func(v,w.p[i]);			  \
  z.temp=true; return z;                                                               \
}

BinaryFunction( atan2 )
BinaryFunction( pow )
BinaryFunction( fmod )
#undef BinaryFunction

#endif //(doxygen exclusion)



//===========================================================================
//
//!@name more efficient basic operations (don't copy in new arrays)
//

#ifdef MT_MSVC
#  ifdef plus
#    undef plus
#  endif
#endif

//! element-wise plus
template<class T>
void plus(MT::Array<T>& x,const MT::Array<T>& y,const MT::Array<T>& z){
  CHECK(y.N==z.N,"must have same size for adding!");
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=y.p[i]+z.p[i];
}

//! element-wise plus with a scalar
template<class T>
void scalarPlus(MT::Array<T>& x,const MT::Array<T>& y,T a){
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=y.p[i]+a;
}

//! element-wise linear combination (plus with scalar factors for each array)
template<class T>
void plus(MT::Array<T>& x,T a,const MT::Array<T>& y,T b,const MT::Array<T>& z){
  CHECK(y.N==z.N,"must have same size for adding!");
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=a*y.p[i]+b*z.p[i];
}

//! element-wise minus
template<class T>
void minus(MT::Array<T>& x,const MT::Array<T>& y,const MT::Array<T>& z){
  CHECK(y.N==z.N,"must have same size for subtracting!");
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=y.p[i]-z.p[i];
}

//! element-wise multiplication
template<class T>
void mult(MT::Array<T>& x,const MT::Array<T>& y,const MT::Array<T>& z){
  CHECK(y.N==z.N,"must have same size for element-wise multiplication!");
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=y.p[i]*z.p[i];
}

//! element-wise division
template<class T>
void div(MT::Array<T>& x,const MT::Array<T>& y,const MT::Array<T>& z){
  CHECK(y.N==z.N,"must have same size for element-wise multiplication!");
  uint i,n=y.N;
  x.resize(y);
  for(i=0;i<n;i++) x.p[i]=y.p[i]/z.p[i];
}


//===========================================================================
//
//!@name matrix operations
//

//! mat becomes dim*dim identity matrix
template<class T>
void setIdentity(MT::Array<T>& mat,uint dim){
  mat.resize(dim,dim); mat=0.;
  for(uint i=0;i<dim;i++) mat(i,i)=1.;
}

//! returns the identity matrix
inline MT::Array<double> Identity(uint dim){
  MT::Array<double> z;
  setIdentity(z,dim);
  return z;
}

//! make symmetric \f$A=(A+A^T)/2\f$
inline void makeSymmetric(MT::Array<double>& A){
  CHECK(A.nd==2 && A.d0==A.d1,"not symmetric");
  uint n=A.d0,i,j;
  for(i=1;i<n;i++) for(j=0;j<i;j++) A(j,i) = A(i,j) = .5 * (A(i,j) + A(j,i));
}

//! x = y^T
template<class T>
void transpose(MT::Array<T>& x,const MT::Array<T>& y){
  CHECK(y.nd<=2,"can only transpose up to 2D arrays");
  if(y.nd==2){
    uint i,j,d0=y.d1,d1=y.d0;
    x.resize(d0,d1);
    for(i=0;i<d0;i++) for(j=0;j<d1;j++) x.p[i*d1+j]=y.p[j*d0+i];
    return;
  }
  if(y.nd==1){
    x=y;
    x.resizeCopy(1,y.N);
    return;
  }
  HALT("transpose not implemented for this dims");
}

//! returns the diagonal x = diag(y) (the diagonal-vector of the symmetric 2D matrix y)
template<class T>
inline void diag(MT::Array<T>& x,const MT::Array<T>& y){
  CHECK(y.nd==2 && y.d0==y.d1,"can only give diagonal of symmetric 2D matrix");
  x.resize(y.d0);
  uint i;
  for(i=0;i<x.d0;i++) x(i)=y(i,i);
}

//! sets x to be the diagonal matrix with diagonal v
template<class T>
inline void setDiagonal(MT::Array<T>& x,const MT::Array<T>& v){
  CHECK(v.nd==1,"can only give diagonal of 1D array");
  x.resize(v.d0,v.d0);
  x.setZero();
  uint i;
  for(i=0;i<v.d0;i++) x(i,i)=v(i);
}

//! x is v times the d-dimensional Identity
template<class T>
inline void setDiagonal(MT::Array<T>& x,uint d,double v){
  x.resize(d,d);
  x.setZero();
  uint i;
  for(i=0;i<d;i++) x(i,i)=v;
}

//! inverse of a 2d matrix
template<class T>
void inverse2d(MT::Array<T>& invA,const MT::Array<T>& A){
  invA.resize(2,2);
  invA(0,0)=A(1,1); invA(1,1)=A(0,0); invA(0,1)=-A(0,1); invA(1,0)=-A(1,0);
  invA/=A(0,0)*A(1,1)-A(0,1)*A(1,0);
}

//! constructs the block matrix X=[A,B ; C,D]
template<class T>
void blockMatrix(MT::Array<T>& X,const MT::Array<T>& A,const MT::Array<T>& B,const MT::Array<T>& C,const MT::Array<T>& D){
  CHECK(A.nd==2 && B.nd==2 && C.nd==2 && D.nd==2,"");
  CHECK(A.d0==B.d0 && A.d1==C.d1 && B.d1==D.d1 && C.d0==D.d0,"");
  uint i,j,a=A.d0,b=A.d1;
  X.resize(A.d0+C.d0,A.d1+B.d1);
  for(i=0;i<A.d0;i++) for(j=0;j<A.d1;j++) X(i  ,j  )=A(i,j);
  for(i=0;i<B.d0;i++) for(j=0;j<B.d1;j++) X(i  ,j+b)=B(i,j);
  for(i=0;i<C.d0;i++) for(j=0;j<C.d1;j++) X(i+a,j  )=C(i,j);
  for(i=0;i<D.d0;i++) for(j=0;j<D.d1;j++) X(i+a,j+b)=D(i,j);
}


//===========================================================================
//
//!@name products
//

//! product with a scalar
template<class T>
void scalarMultiplication(MT::Array<T>& x,const MT::Array<T>& y,const T& a){
  x.resize(y);
  T *ix=x.p;
  const T *iy=y.p;
  for(; ix!=x.pstop; ix++,iy++) *ix = *iy * a;
}

/*!\brief inner product (also ordinary matrix or scalar product):
  \f$\forall_{ik}:~ x_{ik} = \sum_j v_{ij}\, w_{jk}\f$ but also:
  \f$\forall_{i}:~ x_{i} = \sum_j v_{ij}\, w_{j}\f$*/
template<class T> 
void innerProduct(MT::Array<T>& x,const MT::Array<T>& y, const MT::Array<T>& z){
  /*
  if(y.nd==2 && z.nd==2 && y.N==z.N && y.d1==1 && z.d1==1){  //elem-wise
    HALT("make element-wise multiplication explicite!");
    mult(x,y,z);
    return;
  }
  */
  if(y.nd==2 && z.nd==2){ //plain matrix multiplication
    CHECK(y.d1==z.d0,"wrong dimensions for inner product");
    uint i,j,d0=y.d0,d1=z.d1,dk=y.d1;
    T *a,*astop,*b,*c;
    x.resize(d0,d1); x.setZero();
    c=x.p;
    for(i=0;i<d0;i++) for(j=0;j<d1;j++){
      //for(s=0.,k=0;k<dk;k++) s+=y.p[i*dk+k]*z.p[k*d1+j];
      //this is faster:
      a=y.p+i*dk; astop=a+dk; b=z.p+j;
      for(;a<astop; a++, b+=d1) (*c)+=(*a) * (*b);
      c++;
    }
    return;
  }
  if(y.nd==2 && z.nd==1){ //matrix x vector -> vector
    CHECK(y.d1==z.d0,"wrong dimensions for inner product");
    uint i,k,d0=y.d0,dk=y.d1;
    T *a,*b;
    T s;
    x.resize(d0);
    for(i=0;i<d0;i++){
      //for(s=0.,k=0;k<dk;k++) s+=y.p[i*dk+k]*z.p[k];
      //this is faster:
      a=y.p+i*dk; b=z.p;
      for(s=0,k=0;k<dk;k++){ s+=(*a) * (*b); a++; b++; }
      x.p[i]=s;
    }
    return;
  }
  if(y.nd==1 && z.nd==2 && z.d0==1){ //vector x vector^T -> matrix (outer product)
    CHECK(y.d0==z.d1,"wrong dimensions for inner product");
    uint i,j,d0=y.d0,d1=z.d1;
    x.resize(d0,d1);
    for(i=0;i<d0;i++) for(j=0;j<d1;j++) x(i,j)=y(i)*z(0,j);
    return;
  }
  if(y.nd==1 && z.nd==2){ //vector^T x matrix -> vector^T
    CHECK(y.d0==z.d0,"wrong dimensions for inner product");
    uint i,k,d0=z.d1,dk=y.d0;
    x.resize(d0);
    T s;
    for(i=0;i<d0;i++){
      for(s=0,k=0;k<dk;k++) s+=y.p[k]*z.p[k*d0+i];
      x.p[i]=s;
    }
    return;
  }
  if(y.nd==1 && z.nd==1 && z.N==1){ //vector multiplied with scalar (disguised as 1D vector)
    uint k,dk=y.N;
    x.resize(y.N);
    for(k=0;k<dk;k++) x.p[k]=y.p[k]*z.p[0];
    return;
  }
  if(y.nd==1 && z.nd==1){ //should be scalar product, but be careful
    HALT("what do you want? scalar product or element wise multiplication?");
    CHECK(y.d0==z.d0,"wrong dimensions for inner product");
    uint k,dk=y.d0;
    x.resize(1);
    T s;
    for(s=0,k=0;k<dk;k++) s+=y.p[k]*z.p[k];
    x.p[0]=s;
    return;
  }
  HALT("inner product - not yet implemented for these dimensions");
}

/*!\brief outer product (also exterior or tensor product): \f$\forall_{ijk}:~
  x_{ijk} = v_{ij}\, w_{k}\f$ */
template<class T> 
void outerProduct(MT::Array<T>& x,const MT::Array<T>& y, const MT::Array<T>& z){
  if(y.nd==1 && z.nd==1){
    uint i,j,d0=y.d0,d1=z.d0;
    x.resize(d0,d1);
    for(i=0;i<d0;i++) for(j=0;j<d1;j++) x.p[i*d1+j]=y.p[i]*z.p[j];
    return;
  }
  HALT("outer product - not yet implemented for these dimensions");
}

//! \f$\sum_i v_i\, w_i\f$, or \f$\sum_{ij} v_{ij}\, w_{ij}\f$, etc.
template<class T> 
inline T scalarProduct(const MT::Array<T>& v, const MT::Array<T>& w){
  CHECK(v.N==w.N, 
    "scalar product on different array dimensions ("<<v.N<<", "<<w.N<<")");
  T t(0);
  for(uint i=v.N; i--; t+=v.p[i]*w.p[i]);
  return t;
}

//! \f$\sum_{ij} g_{ij}\, v_i\, w_i\f$
template<class T> 
inline T scalarProduct(const MT::Array<T>& g,const MT::Array<T>& v, const MT::Array<T>& w){
  CHECK(v.N==w.N && g.nd==2 && g.d0==v.N && g.d1==w.N,
    "scalar product on different array dimensions ("<<v.N<<", "<<w.N<<")");
  T t(0);
  for(uint i=0;i<g.d0;i++) for(uint j=0;j<g.d1;j++) t+=g(i,j)*v.p[i]*w.p[j];
  return t;
}


//===========================================================================
//
//!@name distances
//

//! \f$\sum_i (v^i-w^i)^2\f$
template<class T> 
inline T sqrDistance(const MT::Array<T>& v, const MT::Array<T>& w){
  CHECK(v.N==w.N, 
    "sqrDistance on different array dimensions ("<<v.N<<", "<<w.N<<")");
  T d,t(0);
  for(uint i=v.N;i--;){ d=v.p[i]-w.p[i]; t+=d*d; }
  return t;
}

//! \f$\sum_{i|{\rm mask}_i={\rm true}} (v^i-w^i)^2\f$
template<class T> 
inline T sqrDistance(const MT::Array<T>& v, const MT::Array<T>& w,const MT::Array<bool>& mask){
  CHECK(v.N==w.N, 
    "sqrDistance on different array dimensions ("<<v.N<<", "<<w.N<<")");
  T d,t(0);
  for(uint i=v.N;i--;) if(mask(i)){ d=v.p[i]-w.p[i]; t+=d*d; }
  return t;
}

//! \f$\sqrt{\sum_{ij} g_{ij} (v^i-w^i) (v^j-w^j)}\f$
template<class T>
inline T sqrDistance(const MT::Array<T>& g,const MT::Array<T>& v,const MT::Array<T>& w){
  MT::Array<T> d;
  minus(d,v,w);
  return scalarProduct(g,d,d);
}

//! \f$\sqrt{\sum_i (v^i-w^i)^2}\f$
template<class T> 
inline T euclideanDistance(const MT::Array<T>& v, const MT::Array<T>& w){
  return ::sqrt(sqrDistance(v,w));
}



//===========================================================================
//
//!@name running sums
//

//! \f$\sum_i x_i\f$
template<class T>
inline T sum(const MT::Array<T>& v){
  T t(0);
  for(uint i=v.N; i--; t+=v.p[i]);
  return t;
}

//! \f$\sum_i |x_i|\f$
template<class T>
inline T sumOfAbs(const MT::Array<T>& v){
  T t(0);
  for(uint i=v.N; i--; t+=fabs(v.p[i]));
  return t;
}

//! \f$\sum_i x_i^2\f$
template<class T>
inline T sumOfSqr(const MT::Array<T>& v){
  T t(0);
  for(uint i=v.N; i--; t+=v.p[i]*v.p[i]);
  return t;
}

//! \f$\sqrt{\sum_i x_i^2}\f$
template<class T>
inline T norm(const MT::Array<T>& v){ return sqrt(sumOfSqr(v)); }

//! \f$\sum_i x_{ii}\f$
template<class T>
inline T trace(const MT::Array<T>& v){
  CHECK(v.nd==2 && v.d0==v.d1,"only for squared matrix");
  T t(0);
  for(uint i=0;i<v.d0;i++) t+=v(i,i);
  return t;
}


//===========================================================================
//
//!@name probability distribution operations
//

/*!\brief entropy \f$H_i = - \sum_x p(X_i=x) \ln p(X_i=x)\f$, where \f$x\f$ ranges
  from \c 0 to \c range-1, of the \c ith variable */
template<class T>
inline T entropy(const MT::Array<T>& v){
  T t(0);
  for(uint i=v.N; i--;) if(v.p[i]) t-=v.p[i]*::log(v.p[i]);
  return t/MT_LN2;
}

//! v = v / sum(v)
template<class T>
inline T normalizeDist(MT::Array<T>& v){
  double Z=sum(v);
  if(Z) v/=Z; else v=1./(double)v.N;
  return Z;
}

template<class T>
inline void eliminate(MT::Array<T>& x,const MT::Array<T>& y,uint d){
  CHECK(y.nd==2,"only implemented for 2D yet");
  uint i,j;
  if(d==1){
    x.resize(y.d0); x=0.;
    for(i=0;i<y.d0;i++) for(j=0;j<y.d1;j++) x(i)+=y(i,j);
  }
  if(d==0){
    x.resize(y.d1); x=0.;
    for(i=0;i<y.d0;i++) for(j=0;j<y.d1;j++) x(j)+=y(i,j);
  }
}

template<class T>
inline void eliminate(MT::Array<T>& x,const MT::Array<T>& y,uint d,uint e){
  CHECK(y.nd==3,"only implemented for 3D yet");
  uint i,j,k;
  if(d==1 && e==2){
    x.resize(y.d0); x=0.;
    for(i=0;i<y.d0;i++) for(j=0;j<y.d1;j++) for(k=0;k<y.d2;k++) x(i)+=y(i,j,k);
  }
  if(d==0 && e==2){
    x.resize(y.d1); x=0.;
    for(i=0;i<y.d0;i++) for(j=0;j<y.d1;j++) for(k=0;k<y.d2;k++) x(j)+=y(i,j,k);
  }
  if(d==0 && e==1){
    x.resize(y.d2); x=0.;
    for(i=0;i<y.d0;i++) for(j=0;j<y.d1;j++) for(k=0;k<y.d2;k++) x(k)+=y(i,j,k);
  }
}


//===========================================================================
//
//!@name running products
//

//! \f$\prod_i x_i\f$
template<class T>
inline T product(const MT::Array<T>& v){
  T t(1);
  for(uint i=v.N; i--; t *= v.p[i]);
  return t;
}


//===========================================================================
//
//!@name min and max
//

//! \f$\min_i x_i\f$
template<class T>
inline T minA(const MT::Array<T>& v){
  CHECK(v.N>0,"");
  T t(v.p[0]);
  for(uint i=1; i<v.N; ++i) if(v.p[i]<t) t=v.p[i];
  return t;
}

//! also returns the index (argmin) \c ind
template<class T>
inline T minA(const MT::Array<T>& v, uint & ind, uint start=0, uint end=0){
  CHECK(v.N>0,"");
  CHECK(v.N>start,"");	
  CHECK(v.N>=end,"");
  CHECK(end>=start,"");
  T t(v(start));
  ind=start;
  if (end==0)	end=v.N;
  for(uint i=start; i<end; ++i) if(v.p[i]<t){
    t  =v.p[i];
    ind=i;
  }
  return t;
}

//! \f$\max_i x_i\f$
template<class T>
inline T maxA(const MT::Array<T>& v){
  CHECK(v.N>0,"");
  T t(v.p[0]);
  for(uint i=1; i<v.N; ++i) if(v.p[i]>t) t=v.p[i];
  return t;
}

//! also returns the index (argmax) \c ind
template<class T>
inline T maxA(const MT::Array<T>& v, uint & ind, uint start=0, uint end=0){
  CHECK(v.N>0,"");
  CHECK(v.N>start,"");
  CHECK(v.N>=end,"");
  CHECK(end>=start,"");
  T t(v(start));
  ind=start;
  if(end==0){
    end=v.N;
  }
  for(uint i=start; i<end; ++i) if(v.p[i]>t){
    t  =v.p[i];
    ind=long(i);
  }
  return t;
}

//! gets the min and max
template<class T>
inline void minmaxA(const MT::Array<T>& v, T& minVal, T& maxVal){
  if(v.N>0){
    minVal=maxVal=v.p[0];
    for(uint i=1; i<v.N; ++i){
      if(v.p[i]<minVal) minVal=v.p[i];
      else if(v.p[i]>maxVal) maxVal=v.p[i];
    }
  }
}

template<class T>
inline T absMax(const MT::Array<T>& x){
  CHECK(x.N>0,"");
  T t(x.p[0]);
  for(uint i=1; i<x.N; ++i) if(fabs(x.p[i])>t) t=fabs(x.p[i]);
  return t;
}


//===========================================================================
//
//!@name randomizations
//

//! Assign all elements of \c a to a uniformly distributed variable in [0,1]
template<class T>
void rndInt(MT::Array<T>& a,int low=0.,int high=1.){
  for(uint i=0;i<a.N;i++) a.p[i]=low+(int)rnd.num(1+high-low);
}

//! Assign all elements of \c a to a uniformly distributed variable in [0,1]
template<class T>
void rndUni(MT::Array<T>& a,double low=0.,double high=1.){
  for(uint i=0;i<a.N;i++) a.p[i]=rnd.uni(low,high);
}

//! Assign all elements of \c a to a Gaussian random variable
template<class T>
void rndGauss(MT::Array<T>& a,double stdDev,bool add=false){
  if(a.nd==2 && a.d0!=a.d1)  //assumes that it is a batch of data
    stdDev/=::sqrt((double)a.d1);
  else
    stdDev/=::sqrt((double)a.N);
  if(!add) for(uint i=0;i<a.N;i++) a.p[i]=rnd.gauss(stdDev);
  else     for(uint i=0;i<a.N;i++) a.p[i]+=rnd.gauss(stdDev);
}

//! returns an array with \c dim Gaussian noise elements
template<class T>
MT::Array<T>& rndGauss(double stdDev,uint dim){
  static MT::Array<T> z;
  stdDev/=::sqrt(dim);
  z.resize(dim);
  rndGauss(z,stdDev);
  return z;
}

/*!\brief from a vector of numbers, calculates the softmax distribution
    soft(i) = exp(beta*a(i)), and returns a random sample from
    this distribution (an index in {0,..,a.N-1}) */
template<class T>
uint softMax(const MT::Array<T>& a,MT::Array<double>& soft,double beta){
  double norm=0.,r;
  uint i; int sel=-1;
  soft.resize(a);
  for(i=0;i<a.N;i++){
    soft(i)=exp(beta*a(i));
    norm+=soft(i);
  }
  r=rnd.uni();
  for(i=0;i<a.N;i++){
    soft(i)/=norm;
    r-=soft(i);
    if(sel==-1 && r<0.) sel=i;
  }
  return sel;
}


//===========================================================================
//
//!@name certain initializations
//

template<class T>
MT::Array<T>& grid(MT::Array<T>& a,uint dim,T lo,T hi,uint steps){
  uint i,j;
  if(dim==1){
    a.resize(steps+1,1);
    for(i=0;i<a.d0;i++) a(i,0)=lo+(hi-lo)*i/steps;
    return a;
  }
  if(dim==2){
    a.resize(steps+1,steps+1,2);
    for(i=0;i<a.d0;i++) for(j=0;j<a.d1;j++){
      a(i,j,0)=lo+(hi-lo)*j/steps;
      a(i,j,1)=lo+(hi-lo)*i/steps;
    }
    a.reshape(a.d0*a.d1,2);
    return a;
  }
  HALT("not implemented yet");
}


//===========================================================================
//
//!@name operators realized as expressions -- PRELIMINARY!
//

#ifdef MT_EXPRESSIONS

inline MT::Array<double> operator*(double a,const MT::Array<double>& b){ return MT::Array<double>(new MT::Ex((void*)&b,0,a,false)); }
inline MT::Array<double> operator*(const MT::Array<double>& a,double b){ return MT::Array<double>(new MT::Ex((void*)&a,0,b,false)); }
inline MT::Array<double> operator/(const MT::Array<double>& a,double b){ return MT::Array<double>(new MT::Ex((void*)&a,0,1./b,false)); }
inline MT::Array<double> operator+(double a,const MT::Array<double>& b){ return MT::Array<double>(new MT::Ex((void*)&b,a,1,false)); }
inline MT::Array<double> operator+(const MT::Array<double>& a,double b){ return MT::Array<double>(new MT::Ex((void*)&a,b,1,false)); }
inline MT::Array<double> operator-(double a,const MT::Array<double>& b){ return MT::Array<double>(new MT::Ex((void*)&b,-a,1,false)); }
inline MT::Array<double> operator-(const MT::Array<double>& a,double b){ return MT::Array<double>(new MT::Ex((void*)&a,-b,1,false)); }
inline MT::Array<double> operator~(const MT::Array<double>& a){ return MT::Array<double>(new MT::Ex((void*)&a,0,1,true)); }

inline MT::Array<double> operator*(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::PROD,(void*)&y,(void*)&z)); }
inline MT::Array<double> operator^(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::OUT ,(void*)&y,(void*)&z)); }
inline MT::Array<double> operator%(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::MUL ,(void*)&y,(void*)&z)); }

inline MT::Array<double> operator/(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::Div,(void*)&y,(void*)&z)); }
inline MT::Array<double> operator+(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::PLUS,(void*)&y,(void*)&z)); }
inline MT::Array<double> operator-(const MT::Array<double>& y,const MT::Array<double>& z){ return MT::Array<double>(new MT::Ex(MT::MINUS,(void*)&y,(void*)&z)); }

void assign(MT::Array<double>& x,const MT::Array<double>& a);
void assign(MT::Array<double>& x);
template<class T> inline void MT::Array<T>::assignEx(const MT::Array<T>& a) const{ ::assign(*((MT::Array<double>*)this),*((MT::Array<double>*)&a)); }
template<class T> inline void MT::Array<T>::assignEx() const{ ::assign(*((MT::Array<double>*)this)); }

#endif //MT_EXPRESSIONS


//===========================================================================
//
//!@name gnuplot fun
//

//! calls gnuplot to display the (n,2) or (n,3) array (n=number of points of line or surface)
void gnuplot(const MT::Array<double>& X);

//void write(const MT::Array<double>& X,MT::Array<double>& Y,const char* name);
//void write(const MT::Array<double>& X,const char* name);

//===========================================================================
//
//!@name permutation class
//

namespace MT{
//! A permutation realized as 1-dimensional array of indices.
class Permutation:public Array<uint>{
private:
  Array<uint> storage;

public:
  //! ...
  Permutation(){ init(0); };

public://!@name Assignment
  //! sorted permutation of length \c n
  void init(uint n){ resize(n); for(uint i=0;i<N;i++) elem(i)=i; }
  //! random permutation of length \c n
  void random(uint n){ init(n); random(); }
  //! reverse sorted permutation of lenth \c N
  void reverse(uint n){ resize(n); for(uint i=0;i<N;i++) elem(N-1-i)=i; }

public://!@name Operations
  //! \c elem(i) = elem(i) + offset modulo permutation range \c ne
  void push(int offset=1){for(uint i=0;i<N;i++) elem(i)=((int)elem(i)+offset)%N;}
  //! store the current permutation
  void store(){ storage.operator=(*this); }
  //! restore the previously stored permutation
  void restore(){ (Array<uint>&)(*this)=storage; }
  //! permute indices of \c i and \c j
  void permute(uint i,uint j){ uint x=elem(i); elem(i)=elem(j); elem(j)=x; }
  //! randomize permutation
  void random(){
    int j,r;
    for(j=N-1;j>=1;j--){
      r=rnd(j+1);
      permute(r,j);
    }
  }
  //! permutes the entries of a according to the current this
  template<class T>
  void permute(Array<T>& a){
    CHECK(N<=a.N,"array smaller than permutation ("<<a.N<<"<"<<N<<")");
    Array<T> b=a;
    for(uint i=0;i<N;i++) a(i)=b(p[i]);
  }
  //! permutes the entries of a according to the inverse of the current this
  template<class T>
  void invpermute(Array<T>& a){
    CHECK(N<=a.N,"array smaller than permutation ("<<a.N<<"<"<<N<<")");
    Array<T> b=a;
    for(uint i=0;i<N;i++) a(p[i])=b(i);
  }
};
}


//===========================================================================
//
//!@name standard types
//

typedef MT::Array<double> doubleA;
typedef MT::Array<uint> uintA;
typedef MT::Array<int> intA;
typedef MT::Array<char> charA;
typedef MT::Array<byte> byteA;
typedef MT::Array<bool> boolA;

//using MT::Array;


//===========================================================================
//
// implementations
//

template<class T> char MT::Array<T>::memMoveInit=-1;
template<class T> int MT::Array<T>::sizeT=-1;

#ifdef MT_IMPLEMENTATION
#  include"array.cpp"
#endif


#endif
